// ****************************************************************************
// copyright (c) 2000-2002 Horst Knorr <hk_classes@knoda.org>
// This file is part of the hk_classes library.
// This file may be distributed and/or modified under the terms of the
// GNU Library Public License version 2 as published by the Free Software
// Foundation and appearing in the file COPYING included in the
// packaging of this file.
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// ****************************************************************************
#ifndef  HK_DSVISIBLE
#define  HK_DSVISIBLE
#include <hk_definitions.h>
#include <hk_visible.h>
#include <hk_datetime.h>
using namespace std;

/**
 *
 *@short base class for visible widgets without column data connection
 *@version 0.2
 *@author Horst Knorr (hk_classes@knoda.org)

 *This is the base class for visible widgets which do not need access to the data,
 *but do have to know whether or not a datasource is enabled and which row is the actual one.
 */

class hk_dsvisible:public hk_visible

{
    friend class hk_datasource;
    public:
        hk_dsvisible(hk_presentation* presentation=NULL);
        virtual         ~hk_dsvisible(void);
        virtual void        set_datasource(hk_datasource* d);
        virtual bool        set_presentationdatasource(long n, bool registerchange=true);
        long        presentationdatasource(void);
        hk_datasource*  datasource(void);
        virtual void savedata(ostream& s, bool savedatasource,bool saveall);
        virtual void savedata(ostream& s  );
        virtual void loaddata(const hk_string& definition);
        void    set_readonly(bool readonly);
        bool    is_readonly(void) const;

    protected:
        void        row_change(void);

        void        insert_mode(void);
/**
 *FOR WIDGET DEVELOPERS ONLY:
 * will be automatically called when the datasource will be set to insert_mode.
 *Has to be rewritten for a new widget
 * @see before_row_change.
 *
 */
        virtual void        widget_specific_insert_mode(void){}
/**
 *FOR WIDGET DEVELOPERS ONLY:
 * will be automatically called when the datasource has changed the row.
 *Has to be rewritten for a new widget
 * @see before_row_change.
 *
 */
        virtual bool        widget_specific_row_change(void) {return false;}
/**
 *FOR WIDGET DEVELOPERS ONLY:
 * defines whether the widget should be enabled or disabled. Has to be rewritten
 * for a new widget
 */
        virtual void        widget_specific_enable_disable(void){}
        virtual void        widget_specific_enable(void){}
        virtual void        widget_specific_disable(void){}
        virtual void        widget_specific_row_added(void){}
        virtual void        widget_specific_row_deleted(void){}
/**
 *FOR WIDGET DEVELOPERS ONLY:
 *called from hk_datasource before it tries to change the row or disable
 *widget can i.e. store changed data there
 */
        virtual void        before_row_change(void){}
        virtual void        before_store_changed_data(void){}
        virtual void        after_store_changed_data(void);
        virtual void        before_insert_row(void){}
        virtual void        before_datasource_enables(void){}
/**
 *FOR WIDGET DEVELOPERS ONLY:
 *will be automatically called when the datasources is disabling
 */
        virtual bool        datasource_disable(void);
/**
 *FOR WIDGET DEVELOPERS ONLY:
 *will be automatically called when the datasources is enabling
 */
        virtual bool        datasource_enable(void);
/**
 *FOR WIDGET DEVELOPERS ONLY:
 *will be automatically called when the datasources is destroyed
 */
        virtual void        datasource_delete(void);
/**
 *FOR WIDGET DEVELOPERS ONLY:
 *will be automatically called when the datasources creates new columns
 *i.e.when enabling,disabling or when a depending datasource has to requery
 *its rows.
 */
        virtual void        row_added(void);
        virtual void        row_deleted(void);
        virtual bool        columns_new_created(void){return true;}
        virtual void        list_changes(filetype type);
/**
 *FOR WIDGET DEVELOPERS ONLY:
 *will be automatically called before the columns will be deleted.
 *usually column pointers will be set to NULL in this function.
 *i.e.when enabling,disabling or when a depending datasource has to requery
 *its rows.
 */

        virtual bool        before_columns_deleted(void){return true;}
/**
 *FOR WIDGET DEVELOPERS ONLY:
 *will be automatically called after the columnstructure of a table
 *was successfully altered.
 */
        virtual bool        table_structure_changes(void){return true;}
/**
 *FOR WIDGET DEVELOPERS ONLY:
 *will be automatically called before the database will be closed.(i.e when a new database will
 *be created and the old one will be destroyed by hk_connection
 */

        virtual void        before_source_vanishes(void);
/**
 *FOR WIDGET DEVELOPERS ONLY:
 *will be automatically called before the presentationdatasource will be set
 */

        virtual void        widget_specific_presentationdatasource(long){}
        hk_datasource*  p_datasource;
        hk_datetime p_date;
#ifdef HK_DEBUG
        hk_string   debug_p_columnname;
#endif
    private:
        long        p_presentationdatasource;
        bool        internal_dsrefresh;
        mutable bool        p_readonly;
//p_already_handled  is used from hk_datasource in bulk operations to find out if this object has been already handled.
        bool    p_already_handled;

};
#endif
